/* 

Copyright 2006: Irmtraud M. Meyer (irmtraud.meyer@cantab.net) and Istvan Miklos 

This file is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation.
See http://www.gnu.org/copyleft/gpl.html for more information.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _global_h
#define _global_h

#include "list.h"

#undef NULL
#define NULL 0L

#define LOOP_MIN 0 /* minimal loopsize for hairpin Irmi set to 3 */
#define INC 10     /* default increment to increase memory of array */
#define MAXITER 30 /* max number of iterations to reach structure convergence */

typedef struct {

    unsigned short bi; /* sequence position of left pairing partner */
    unsigned short bj; /* sequence position of right pairing partner */
    int wt;            /* weight */
    float owt;         /* original weight */
    short xx;          /* 0=isolated, 1=internal, 2=beginning, 3=end */
    int hx;            /* helix number of bp */
    int col;           /* color of bp */

} INFO;

typedef struct _helix {

    short iL;    /* # of bps in helix */
    short bp[2]; /* bp[0]=i; bp[1]=j; with i<j */
    int   iW;    /* weight of helix */
    list *pairs;
} HELIX;

typedef struct _op {
  int pair;
  int helix;
} OP;

typedef struct _stack {
    int bi, bj;         /* coordinates of outer base-pair of helix */
    int len, wt, col;   /* length, weight, colour of helix */
} STACK;

extern int iLMin; /* minimum helix length */
extern int iTMin; /* minimum bp weight */

extern int iV;
extern int iE;
extern INFO* infBpairs;

extern char cOutfile[3000];

extern int iL_helList;
extern const int iL_inc;
extern HELIX* helList;

extern int iH;  /* # of helices */
extern int* iIndx;
extern INFO** infPindx;

#endif 
