#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "global.h"
#include "io.h"
#include "xutils.h"

/* 

Copyright 2006: Irmtraud M. Meyer (irmtraud.meyer@cantab.net) and Istvan Miklos 

This file is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation.
See http://www.gnu.org/copyleft/gpl.html for more information.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

FILE* ReadInputArguments(const int argc, const char** argv) {

    FILE* fFile = NULL;
    int i;
  
    for (i = 1; i < argc - 1 ; i++) { /* read all but first and last argument */
	if (argv[i][0]=='-') {
	    switch(argv[i][1]) {
		case 'h':
		    HowTo();
		    break;
		default:
		    HowTo();
	    }
	}
    }
    /* read last argument [= input file name] and open file */

    if (!(fFile = fopen (argv[argc-1], "r"))) 
	fatal("ERROR: ReadInputArguments: cannot open input file.\n");	  

    return (fFile);
}

int HowTo(void) {

    fprintf(stderr, "\nbp2bistruc in.file > out.file\n");
    return(1);
}

void ReadInfoFromUpperTriangle (FILE* fIn) {

    /* NOTE: sequence positions start at 1 not 0 */

    /* input format of ut-file must be: L = number of nodes = iV
       
    L
    (1,1) (1,2) .... (1,L)
    (2,2) (2,3) .. (2,L)
    ...
    (L,L)

    */

    int i, j, iNBp, iValue;
    float fValue; 

    /* read iV = # of nodes */
    
    fscanf(fIn,"%d", &iV); 
#ifdef _DEBUG
    printf("ReadInfoFromUpperTriangle iV = %d\n", iV);
#endif
    /* read iE = # of edges = # of bps with weight > 0 */

    iNBp = 0;
    while (getc(fIn)!='\n') ;
    for (j = 1; j <= iV; ++j) {
	for (i = 1; i <= iV-j+1; ++i) {
	    fscanf(fIn,"%f", &fValue);
	    if (fValue > 0.0) {iNBp++;}
	}
	while (getc(fIn)!='\n') ;
    }
    iE = iNBp;
#ifdef _DEBUG
    printf("ReadInfoFromUpperTriangle iE = %d \n", iE);
#endif
    /* go back to start of file */

    rewind(fIn); 

    /* allocate memory for array INFO* bpairs of length iNBp */

    if (infBpairs) {free(infBpairs);}
    infBpairs = NULL;
    if(!(infBpairs = (INFO *)calloc(iNBp, sizeof(INFO)))) {
	fatal("ERROR: ReadInfoFromUpperTriangle: allocation of INFO* infBpairs failed.");
    }
    
    /* fill array INFO* infBpairs with one entry for each bp with weight > 0 */

    iNBp = 0;
    while (getc(fIn)!='\n'); /* skip first line of file */
    for (j = 1; j <= iV; ++j) {
	for (i = 1; i <= iV-j+1; ++i) {
	    fscanf(fIn,"%f", &fValue);
	    if (fValue > 0.0) {
		iValue = (int) (fValue * 100000.0); /* Irmi: make scale value a global variable */
		infBpairs[iNBp].bi  = j;
		infBpairs[iNBp].bj  = i+j-1;
		infBpairs[iNBp].wt  = iValue;
		infBpairs[iNBp].owt = fValue;
#ifdef _DEBUG
		printf("-> %d \t %d %d %d %f pointer %d\n", 
		       iNBp, infBpairs[iNBp].bi, infBpairs[iNBp].bj, infBpairs[iNBp].wt, 
		       infBpairs[iNBp].owt, &infBpairs[iNBp]);
#endif
		iNBp++;
	    }
	}
	while (getc(fIn)!='\n');
    }

    /* close file */

    fclose(fIn); 

    return;
}

char* PrintStructure(const int iNodes, const int iNBP, const INFO *infBP, char* oldString, int* iChanged) { 
		     
    // print bps with color 1 and 2 and status != 0 to return string newString
    // (*iChanged) is 1 if input oldString was NULL or if String changed, otherwise it is 0
    
    int i, ii, ij;
    char* newString;

    /* get newString */

    newString = NULL;
    if (!(newString = (char *) calloc(iNodes, sizeof(char))))
	fatal("ERROR: PrintStructure: memory allocation failed.\n");
  
    for (i=0; i<iNodes-1; i++)
	newString[i] = '.';

    for (i=0; i<iNBP; i++) {
	if (infBP[i].xx) {
	    ii = infBP[i].bi-1;
	    ij = infBP[i].bj-1;
	    switch (infBP[i].col) {
		case 1:
		    newString[ii] = '(';
		    newString[ij] = ')';
		    break;
		case 2:      
		    newString[ii] = '[';
		    newString[ij] = ']';
		    break;
		default:
		    break;
	    }
	}
    } 

    /* compare oldString and newString, release memory of oldString */

    (*iChanged) = 1;

    if (oldString) {

	printf("PrintStructure: old string exists.\n");

	(*iChanged) = 0;
	for (i=0; i<iNodes; i++) {
	    if (newString[i] != oldString[i]) {
		(*iChanged) = 1;
		break;
	    }
	}
	free(oldString);
    }

#ifdef _DEBUG
    printf("PrintStructure: new string %s\n", newString);
#endif
    return(newString);
} 
