/* 

Copyright 2006: Irmtraud M. Meyer (irmtraud.meyer@cantab.net) and Istvan Miklos 

This file is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation.
See http://www.gnu.org/copyleft/gpl.html for more information.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// compile with: "make bp2bistruc"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// non-mwm

#include "global.h"
#include "io.h"
#include "process.h"

// mwm

#include "graphtypes.h"
#include "glib.h"
#include "wmatch.h"

int main (int argc, char **argv) {

    FILE* fIn; 
    int i, j, iOriginalE;
    INFO* infOriginalBP = NULL;    
    Graph graph = NULL;
    int* iMate = NULL;
    char* cString = NULL;

    fIn = ReadInputArguments(argc, argv); 

    ReadInfoFromUpperTriangle(fIn);

    iOriginalE = iE;

    CopyBpairs(iE, infBpairs, &iOriginalE, &infOriginalBP);

    ModifyBPWeights();

    graph = ConvertBpairsToGraph(iV, iE, infBpairs);

    iMate = Weighted_Match(graph, 1, 1);

#ifdef _DEBUG    
    for (i=1; i<=iV; i++) { 
	if (iMate[i] > 0) {
	    printf("MWM %d-%d\n",i, iMate[i]);
	}
    }
#endif	

    PostProcess(iV, iMate, &iE, &infBpairs, &cString);

    GetOriginalWeightofRefBP(iOriginalE, infOriginalBP, &iE, &infBpairs);

    printf("Sequence positions start at 1.\n");
    printf("structure %s\n", cString);
    for (i=0; i<iE; i++) { 
	printf("%d\t%d\t%f\n", infBpairs[i].bi, infBpairs[i].bj, infBpairs[i].owt);
    }

    /* free memory */
    
    if (cString) free (cString);
    cString = NULL;

    if (iMate) free(iMate);
    iMate = NULL;

    if (graph) free(graph);
    graph = NULL;

    if (infOriginalBP) free(infOriginalBP);
    infOriginalBP = NULL;

    if (infBpairs) free(infBpairs);
    infBpairs = NULL;

    return 0;
}

/* End of file */
